@@ -71,6 +71,8 @@ extern "C" {
 // that there's a better replacement in the newer deadbeef versions.
 
 // API version history:
+// 1.17 -- deadbeef-1.9.6
+// 1.16 -- deadbeef-1.9.4
 // 1.15 -- deadbeef-1.9.0
 // 1.14 -- deadbeef-1.8.8
 // 1.12 -- deadbeef-1.8.4
@@ -99,7 +101,7 @@ extern "C" {
 // 0.1 -- deadbeef-0.2.0
 
 #define DB_API_VERSION_MAJOR 1
-#define DB_API_VERSION_MINOR 16
+#define DB_API_VERSION_MINOR 17
 
 #if defined(__clang__)
 
@@ -136,6 +138,12 @@ extern "C" {
 #define DDB_API_LEVEL DB_API_VERSION_MINOR
 #endif
 
+#if (DDB_WARN_DEPRECATED && DDB_API_LEVEL >= 17)
+#define DEPRECATED_117 DDB_DEPRECATED("since deadbeef API 1.17")
+#else
+#define DEPRECATED_117
+#endif
+
 #if (DDB_WARN_DEPRECATED && DDB_API_LEVEL >= 16)
 #define DEPRECATED_116 DDB_DEPRECATED("since deadbeef API 1.16")
 #else
@@ -467,9 +475,6 @@ typedef struct DB_conf_item_s {
     struct DB_conf_item_s *next;
 } DB_conf_item_t;
 
-// event callback type
-typedef int (*DB_callback_t)(ddb_event_t *, uintptr_t data);
-
 // events
 enum {
     DB_EV_NEXT = 1, // switch to next track
@@ -513,6 +518,13 @@ enum {
     DB_EV_FOCUS_SELECTION = 24, 
 #endif
 
+#if (DDB_API_LEVEL >= 17)
+    // Notify about playback state change,
+    // which includes a switch to another output plugin.
+    // p1 contains the new state.
+    DB_EV_PLAYBACK_STATE_DID_CHANGE = 25,
+#endif
+
     // -----------------
     // structured events
 
@@ -635,7 +647,6 @@ enum ddb_sys_directory_t {
 
 // typecasting macros
 #define DB_PLUGIN(x) ((DB_plugin_t *)(x))
-#define DB_CALLBACK(x) ((DB_callback_t)(x))
 #define DB_EVENT(x) ((ddb_event_t *)(x))
 #define DB_PLAYITEM(x) ((DB_playItem_t *)(x))
 
@@ -655,9 +666,19 @@ typedef struct {
     int samplerate;
     uint32_t channelmask;
     int is_float; // bps must be 32 if this is true
+#if (DDB_API_LEVEL >= 17)
+    uint32_t flags;
+#else
     int is_bigendian;
+#endif
 } ddb_waveformat_t;
 
+#if (DDB_API_LEVEL >= 17)
+enum {
+    DDB_WAVEFORMAT_FLAG_IS_DOP = 0x01,
+};
+#endif
+
 // since 1.5
 #if (DDB_API_LEVEL >= 5)
 
@@ -727,7 +748,7 @@ typedef struct ddb_file_found_data_s {
 #endif
 
 // context for title formatting interpreter
-typedef struct {
+typedef struct ddb_tf_context_s {
     int _size; // must be set to sizeof(tf_context_t)
     uint32_t flags; // DDB_TF_CONTEXT_ flags
     ddb_playItem_t *it; // track to get information from, or NULL
@@ -759,6 +780,9 @@ typedef struct {
     // It's used to determine whether the text needs to be searched for the corresponding esc sequences
     int dimmed;
 #endif
+#if (DDB_API_LEVEL >= 17)
+    void (*metadata_transformer)(struct ddb_tf_context_s *ctx, char *data, size_t size);
+#endif
 } ddb_tf_context_t;
 #endif
 
@@ -920,7 +944,7 @@ typedef struct {
     // It doesn't need to be called if you save the playlist via direct call to `plt_save_*`, or `pl_save_current`
     void (*plt_modified) (ddb_playlist_t *handle);
 
-    // returns modication index
+    // returns modification index
     // the index is incremented by 1 every time playlist changes
     int (*plt_get_modification_idx) (ddb_playlist_t *handle);
 
@@ -2283,7 +2307,7 @@ typedef struct {
     const char *(*source_name) (void);
 
     /// Create a media source instance. It must be freed after use by calling the @c free_source
-    /// @param source_path: a unique name to identify the instance, this will be used to prefix individual instance configuration files, caches, etc.
+    /// @param source_path a unique name to identify the instance, this will be used to prefix individual instance configuration files, caches, etc.
     ddb_mediasource_source_t (*create_source) (const char *source_path);
 
     /// Free the @c source created by @c create_source
@@ -2315,7 +2339,7 @@ typedef struct {
     const char *(*selector_name) (ddb_mediasource_source_t source, ddb_mediasource_list_selector_t selector);
 
     /// Add event listener. Your callback function will be called every time some event occurs. Such as state change, content update, and so on.
-    /// The callback funtion may be executed on background thread, so make sure to dispatch to main to update UI.
+    /// The callback function may be executed on background thread, so make sure to dispatch to main to update UI.
     int (*add_listener) (ddb_mediasource_source_t source, ddb_medialib_listener_t listener, void *user_data);
 
     /// Remove event listener
